<?php

namespace Drupal\twig_ui\Theme;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Theme\Registry as CoreRegistry;

/**
 * An extended Registry class for retrieving the unmodified registry.
 *
 * Twig UI can retrieve template code from the file system for a given theme
 * (including inherited templates). In order to do so, it needs to be able to
 * 1) set the theme for the registry build and 2) bypass the
 * theme_registry:[theme] cache.
 *
 * It's not possible to decorate the theme.registry service.
 * See https://www.drupal.org/project/drupal/issues/3155536.
 */
class ImmutableRegistry extends CoreRegistry {

  /**
   * Get the theme for this instance of Registry.
   *
   * @return string
   *   The machine name of the theme.
   */
  public function getTheme() {
    return $this->theme->getName();
  }

  /**
   * Set the theme for this instance of Registry.
   *
   * @param string $theme_name
   *   The machine name of the theme.
   */
  public function setTheme($theme_name) {
    $this->themeName = $theme_name;
  }

  /**
   * Returns the complete theme registry by rebuilding it.
   *
   * This method's code is identical to the parent's method with the exception
   * that cached values are stored in twig_ui.theme_registry:[theme] instead of
   * theme_registry:[theme].
   *
   * @return array
   *   The complete theme registry data array.
   *
   * @see Registry::$registry
   */
  public function get() {
    $this->init($this->themeName);
    if (isset($this->registry[$this->theme->getName()])) {
      return $this->registry[$this->theme->getName()];
    }
    if ($cache = $this->cache->get('twig_ui.theme_registry:' . $this->theme->getName())) {
      $this->registry[$this->theme->getName()] = $cache->data;
    }
    else {
      $this->build();
      // Only persist it if all modules are loaded to ensure it is complete.
      if ($this->moduleHandler->isLoaded()) {
        $this->setCache();
      }
    }
    return $this->registry[$this->theme->getName()];
  }

  /**
   * {@inheritdoc}
   *
   * This method's code is identical to the parent's method with the exception
   * that cached values are stored in twig_ui.theme_registry:[theme] instead of
   * theme_registry:[theme].
   */
  protected function setCache() {
    $this->cache->set('twig_ui.theme_registry:' . $this->theme->getName(), $this->registry[$this->theme->getName()], Cache::PERMANENT, ['theme_registry']);
  }

}
